# fbdfix.py

"""Remove data containing non-frame-markers from FLIMbox FBD files.

Usage: python fbdfix.py directory_or_file_name

"""

import os

import lfdfiles
import numpy


def fbdfix(
    fbdfile,
    outfile=None,
    nmarkers=3,
    frame_duration=None,
    frame_duration_maxdiff=None,
):
    """Remove data containing non-frame-markers from FLIMbox FBD file."""
    if outfile is None:
        dirname, fname = os.path.split(fbdfile)
        outfile = os.path.join(dirname, '_' + fname)

    with lfdfiles.FlimboxFbd(fbdfile) as fbd:
        bins, times, markers = fbd.decode()

    frame_durations = numpy.diff(times[markers]).astype(numpy.int64)
    if len(frame_durations) <= nmarkers:
        return 'skipped'

    if frame_duration is None:
        # assume longest of last few frames is good
        frame_duration = numpy.max(frame_durations[-nmarkers:])
    if frame_duration_maxdiff is None:
        frame_duration_maxdiff = frame_duration // 512

    n = nmarkers
    i = len(frame_durations)
    while i > 0:
        i -= 1
        if abs(frame_durations[i] - frame_duration) > frame_duration_maxdiff:
            n -= 1
            if n == 0:
                break
        else:
            n = nmarkers
    else:
        i = -nmarkers

    i += nmarkers
    skip_bytes = max(0, markers[i] * 2 - 4)

    with open(fbdfile, 'rb') as fh:
        header = fh.read(32 * 1024)
        fh.seek(skip_bytes, 1)
        data = fh.read()

    with open(outfile, 'wb') as fh:
        fh.write(header)
        fh.write(data)

    return f'removed {skip_bytes} bytes, {i} of {markers.size} markers'


def main():
    """Command line script main function."""
    import glob
    import sys

    if len(sys.argv) == 2:
        arg = sys.argv[1]
        if '*' in arg:
            files = glob.glob(arg, recursive=True)
        elif os.path.isdir(arg):
            files = glob.glob(os.path.join(arg, '**/*.fbd'), recursive=True)
        else:
            files = [arg]
    else:
        files = []

    if not files:
        print(__doc__)
        return

    if len(files) > 1:
        common = len(os.path.commonpath(files))
        if files[0][common] == os.path.sep:
            common += 1
        if common > 0:
            print(files[0][:common])
    else:
        common = 0

    for filename in files:
        fname = os.path.split(filename)[-1]
        if fname.startswith('_') or not fname.lower().endswith('.fbd'):
            continue
        print()
        print('*', filename[common:])
        try:
            print(' ', fbdfix(filename))
        except Exception as exc:
            print(f'  failed: {exc.__class__.__name__}: {exc}')


if __name__ == '__main__':
    main()

# mypy: allow-untyped-defs, allow-untyped-calls



# lfdfiles/setup.py

"""Lfdfiles package Setuptools script."""

import re
import sys

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext as _build_ext

buildnumber = ''


def search(pattern, string, flags=0):
    """Return first match of pattern in string."""
    match = re.search(pattern, string, flags)
    if match is None:
        raise ValueError(f'{pattern!r} not found')
    return match.groups()[0]


def fix_docstring_examples(docstring):
    """Return docstring with examples fixed for GitHub."""
    start = True
    indent = False
    lines = ['..', '  This file is generated by setup.py', '']
    for line in docstring.splitlines():
        if not line.strip():
            start = True
            indent = False
        if line.startswith('>>> '):
            indent = True
            if start:
                lines.extend(['.. code-block:: python', ''])
                start = False
        lines.append(('    ' if indent else '') + line)
    return '\n'.join(lines)


with open('lfdfiles/lfdfiles.py', encoding='utf-8') as fh:
    code = fh.read()

version = search(r"__version__ = '(.*?)'", code).replace('.x.x', '.dev0')
version += ('.' + buildnumber) if buildnumber else ''

description = search(r'"""(.*)\.(?:\r\n|\r|\n)', code)

readme = search(
    r'(?:\r\n|\r|\n){2}"""(.*)"""(?:\r\n|\r|\n){2}from __future__',
    code,
    re.MULTILINE | re.DOTALL,
)
readme = '\n'.join(
    [description, '=' * len(description)] + readme.splitlines()[1:]
)

if 'sdist' in sys.argv:
    # update README, LICENSE, and CHANGES files

    with open('README.rst', 'w', encoding='utf-8') as fh:
        fh.write(fix_docstring_examples(readme))

    license = search(
        r'(# Copyright.*?(?:\r\n|\r|\n))(?:\r\n|\r|\n)+""',
        code,
        re.MULTILINE | re.DOTALL,
    )
    license = license.replace('# ', '').replace('#', '')

    with open('LICENSE', 'w', encoding='utf-8') as fh:
        fh.write('BSD 3-Clause License\n\n')
        fh.write(license)

    revisions = search(
        r'(?:\r\n|\r|\n){2}(Revisions.*)- …',
        readme,
        re.MULTILINE | re.DOTALL,
    ).strip()

    with open('CHANGES.rst', encoding='utf-8') as fh:
        old = fh.read()

    d = revisions.splitlines()[-1]
    old = old.split(d)[-1]
    with open('CHANGES.rst', 'w', encoding='utf-8') as fh:
        fh.write(revisions.strip())
        fh.write(old)


class build_ext(_build_ext):
    """Delay import numpy until build."""

    def finalize_options(self):
        _build_ext.finalize_options(self)
        if isinstance(__builtins__, dict):
            __builtins__['__NUMPY_SETUP__'] = False
        else:
            setattr(__builtins__, '__NUMPY_SETUP__', False)
        import numpy

        self.include_dirs.append(numpy.get_include())


# Work around "Cython in setup_requires doesn't work"
# https://github.com/pypa/setuptools/issues/1317
try:
    import Cython  # noqa

    ext = '.pyx'
except ImportError:
    ext = '.c'

if sys.platform == 'win32':
    extra_compile_args = ['/openmp']
    extra_link_args: list[str] = []
elif sys.platform == 'darwin':
    # https://mac.r-project.org/openmp/
    extra_compile_args = ['-Xclang', '-fopenmp']
    extra_link_args = ['-lomp']
else:
    extra_compile_args = ['-fopenmp']
    extra_link_args = ['-fopenmp']

ext_modules = [
    Extension(
        'lfdfiles._lfdfiles',
        ['lfdfiles/_lfdfiles' + ext],
        extra_compile_args=extra_compile_args,
        extra_link_args=extra_link_args,
        define_macros=[
            # ('CYTHON_TRACE_NOGIL', '1'),
            # ('CYTHON_LIMITED_API', '1'),
            # ('Py_LIMITED_API', '1'),
            ('NPY_NO_DEPRECATED_API', 'NPY_1_7_API_VERSION'),
        ],
    ),
]

setup(
    name='lfdfiles',
    version=version,
    license='BSD',
    description=description,
    long_description=readme,
    long_description_content_type='text/x-rst',
    author='Christoph Gohlke',
    author_email='cgohlke@cgohlke.com',
    url='https://www.cgohlke.com',
    project_urls={
        'Bug Tracker': 'https://github.com/cgohlke/lfdfiles/issues',
        'Source Code': 'https://github.com/cgohlke/lfdfiles',
        # 'Documentation': 'https://',
    },
    python_requires='>=3.9',
    install_requires=['numpy', 'tifffile', 'click'],
    setup_requires=['setuptools', 'numpy'],
    extras_require={
        'all': [
            'imagecodecs',
            'matplotlib',
            'czifile',
            'oiffile',
            'netpbmfile',
        ],
        'test': ['pytest'],
    },
    packages=['lfdfiles'],
    package_data={'lfdfiles': ['py.typed']},
    entry_points={
        'console_scripts': [
            'lfdfiles = lfdfiles.__main__:main',
            'fbd2b64 = lfdfiles.fbd2b64:main',
        ]
    },
    ext_modules=ext_modules,
    cmdclass={'build_ext': build_ext},
    zip_safe=False,
    platforms=['any'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Cython',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    ],
)
